import { Model } from "mongoose";
import { Course, CourseDocument } from "src/schemas/course.schema";
import { UserService } from "./user.service";
import { LessonService } from "./lesson.service";
import { CloudinaryService } from "./cloudinary.service";
import { CourseRoutineService } from "./courseRoutine.service";
import { EnrollmentsService } from "./enrollments.service";
import { SectionService } from "./section.service";
export declare class CourseService {
    private courseModel;
    private readonly userService;
    private readonly courseRoutineService;
    private readonly lessonService;
    private readonly cloudinaryService;
    private readonly enrollmentsService;
    private readonly sectionService;
    constructor(courseModel: Model<CourseDocument>, userService: UserService, courseRoutineService: CourseRoutineService, lessonService: LessonService, cloudinaryService: CloudinaryService, enrollmentsService: EnrollmentsService, sectionService: SectionService);
    create(course: Course): Promise<Course>;
    getAllCourses(): Promise<Course[]>;
    update(courseId: string, updatedCourse: Course): Promise<Course>;
    getCourseById(courseId: string, studentId?: string | null): Promise<any>;
    getCoursesByOrganizationId(organizationId: string): Promise<Course[]>;
    getCoursesByStatusAndType(status?: string, courseType?: string): Promise<Course[]>;
    getCoursesByInstructorId(instructorId: string): Promise<Course[]>;
    getCoursesByType(courseType: string): Promise<Course[]>;
    delete(courseId: string): Promise<void>;
    checkCourseNameExists(name: string, courseId?: string): Promise<Course>;
    getTotalCourses(courseType: string): Promise<number>;
    getTotalActiveCourses(courseType: string): Promise<number>;
    getTotalPendingCourses(courseType: string): Promise<number>;
    getTotalFreeCourses(courseType: string): Promise<number>;
    getTotalPaidCourses(courseType: string): Promise<number>;
}
