"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.QuestionMarksService = void 0;
const common_1 = require("@nestjs/common");
const mongoose_1 = require("@nestjs/mongoose");
const mongoose_2 = require("mongoose");
const questionMarks_schema_1 = require("../schemas/questionMarks.schema");
const cloudinary_service_1 = require("./cloudinary.service");
const stream_1 = require("stream");
let QuestionMarksService = class QuestionMarksService {
    constructor(questionMarksModel, cloudinaryService) {
        this.questionMarksModel = questionMarksModel;
        this.cloudinaryService = cloudinaryService;
    }
    async create(questionMarks) {
        if (questionMarks.assignmentFile) {
            const thumbnailStream = new stream_1.Readable();
            thumbnailStream.push(Buffer.from(questionMarks.assignmentFile, "base64"));
            thumbnailStream.push(null);
            const url = await this.cloudinaryService.uploadImage(thumbnailStream);
            questionMarks.assignmentFile = url;
        }
        const createdQuestionMarks = new this.questionMarksModel(questionMarks);
        return createdQuestionMarks.save();
    }
    async edit(questionMarksId, updatedQuestionMarks) {
        const existingQuestionMarks = await this.questionMarksModel.findByIdAndUpdate(questionMarksId, updatedQuestionMarks, {
            new: true
        }).exec();
        if (!existingQuestionMarks) {
            throw new common_1.NotFoundException(`QuestionMarks with ID ${questionMarksId} not found`);
        }
        return existingQuestionMarks;
    }
    async getById(questionMarksId) {
        const questionMarks = await this.questionMarksModel.findById(questionMarksId).exec();
        if (!questionMarks) {
            throw new common_1.NotFoundException(`QuestionMarks with id ${questionMarksId} not found`);
        }
        return questionMarks;
    }
    async getByStudentId(studentId) {
        return this.questionMarksModel.find({ studentId }).exec();
    }
    async getByStudentAndCourseId(courseId, studentId) {
        return this.questionMarksModel.find({ courseId, studentId, questionType: 'QUIZ' }).exec();
    }
    async getByStudentAndLessonId(lessonId, studentId) {
        return this.questionMarksModel.find({ lessonId, studentId }).exec();
    }
    async getByTo10Students(courseId) {
        const courseIdObjectId = new mongoose_2.default.Types.ObjectId(courseId);
        const topStudentsAggregate = await this.questionMarksModel.aggregate([
            {
                $match: { courseId: courseIdObjectId }
            },
            {
                $group: {
                    _id: "$studentId",
                    studentName: { $first: "$studentName" },
                    mark: { $sum: "$mark" },
                    lessonCount: { $addToSet: "$lessonId" }
                }
            },
            {
                $addFields: {
                    lessonCount: { $size: "$lessonCount" }
                }
            },
            {
                $sort: { mark: -1 }
            },
            {
                $limit: 10
            }
        ]);
        if (!topStudentsAggregate || topStudentsAggregate.length === 0) {
            throw new common_1.NotFoundException(`QuestionMarks for course with ID ${courseId} not found`);
        }
        return topStudentsAggregate;
    }
    async getDetailsQuestionMarksByStudentId(studentId) {
        const questionMarks = await this.questionMarksModel
            .find({ studentId }).populate('lessonId').populate('assignmentId').exec();
        if (!questionMarks || questionMarks.length === 0) {
            throw new common_1.NotFoundException(`QuestionMarks for student with ID ${studentId} not found`);
        }
        const output = questionMarks.map((qm) => ({
            _id: qm._id,
            totalQuestion: qm.totalQuestion,
            mark: qm.mark,
            totalMark: qm.totalMark,
            negativeMark: qm.negativeMark,
            totalWrong: qm.totalWrong,
            totalCorrect: qm.totalCorrect,
            questionType: qm.questionType,
            studentId: qm.studentId,
            studentName: qm.studentName,
            lessonId: qm.lessonId._id,
            lessonName: qm.lessonName,
            courseId: qm.courseId,
            assignmentId: qm.assignmentId._id,
            lesson: qm.lessonId,
            assignment: qm.assignmentId,
        }));
        return output;
    }
    async delete(questionMarksId) {
        await this.getById(questionMarksId);
        await this.questionMarksModel.findByIdAndDelete(questionMarksId).exec();
    }
};
QuestionMarksService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, mongoose_1.InjectModel)(questionMarks_schema_1.QuestionMarks.name)),
    __metadata("design:paramtypes", [mongoose_2.Model,
        cloudinary_service_1.CloudinaryService])
], QuestionMarksService);
exports.QuestionMarksService = QuestionMarksService;
//# sourceMappingURL=questionMarks.service.js.map